from ._utils import *

from ..models import RelatedContent
from ..models import RelatedType


def relate(type, source, destination):
    return RelatedContent.objects.create(
        related_type=type,
        source_object=source,
        destination_object=destination)


def create_one_source_to_two_destinations():
    one, two, three = generate_fake_articles(3)
    t = RelatedType.objects.create(title="articles")
    relate(t, one, two)
    relate(t, one, three)
    return one, two, three


def create_two_sources_to_one_destination():
    one, two, three = generate_fake_articles(3)
    t = RelatedType.objects.create(title="articles")
    relate(t, one, two)
    relate(t, three, two)
    return one, two, three


class RelatedContentManagerTestCase(TestCase):
    def test_can_find_by_destination_object(self):
        one, two, three = create_one_source_to_two_destinations()
        related_content = RelatedContent.objects.filter(destination_object=two)
        self.assertEqual(1, related_content.count())

    def test_can_find_by_source_object(self):
        one, two, three = create_two_sources_to_one_destination()
        related_content = RelatedContent.objects.filter(source_object=one)
        self.assertEqual(1, related_content.count())

    def test_by_type_returns_an_empty_queryset_if_no_match_is_found(self):
        one, two = generate_fake_articles(2)

        self.assertEqual([], list(one.related.by_type("unknown")))

    def test_by_type_returns_matching_related_values_by_type_title(self):
        one, two = generate_fake_articles(2)

        t = RelatedType.objects.create(title="articles")
        relate(t, one, two)

        destination_objects = [a.destination_object for a in
                one.related.by_type("articles")]
        self.assertTrue(two in destination_objects)

    def test_by_type_can_be_used_after_filtering(self):
        one, two = generate_fake_articles(2)

        t = RelatedType.objects.create(title="articles")
        relate(t, one, two)

        destination_objects = [a.destination_object for a in
                one.related.all().by_type("articles")]
        self.assertTrue(two in destination_objects)

    def test_uses_queryset_with_custom_filter(self):
        one, two, three = create_one_source_to_two_destinations()
        related_content = (RelatedContent.objects.all()
                .filter(destination_object=two))
        self.assertEqual(1, related_content.count())

    def test_provides_method_for_finding_by_destination(self):
        one, two, three = create_one_source_to_two_destinations()
        related_content = RelatedContent.objects.by_destination(two)
        self.assertEqual(1, related_content.count())
        self.assertEqual(two, related_content[0].destination_object)

    def test_can_call_by_destination_after_filtering(self):
        one, two, three = create_one_source_to_two_destinations()
        related_content = RelatedContent.objects.all().by_destination(two)
        self.assertEqual(1, related_content.count())
        self.assertEqual(two, related_content[0].destination_object)

    def test_provides_method_for_finding_by_source(self):
        one, two, three = create_two_sources_to_one_destination()
        related_content = RelatedContent.objects.by_source(one)
        self.assertEqual(1, related_content.count())
        self.assertEqual(one, related_content[0].source_object)

    def test_can_call_by_source_after_filtering(self):
        one, two, three = create_two_sources_to_one_destination()
        related_content = RelatedContent.objects.all().by_source(one)
        self.assertEqual(1, related_content.count())
        self.assertEqual(one, related_content[0].source_object)



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import query

get_for_model = ContentType.objects.get_for_model


class RelatedContentQuerySet(query.QuerySet):
    def filter(self, destination_object=None, source_object=None, **kwargs):
        """
        See ``QuerySet.filter`` for full documentation

        This adds support for ``destination_object`` and ``source_object``
        as kwargs.  This converts those objects into the values necessary
        to handle the ``GenericForeignKey`` fields.
        """
        if destination_object:
            kwargs.update({
                "destination_id": destination_object.pk,
                "destination_type": get_for_model(destination_object),
            })
        if source_object:
            kwargs.update({
                "source_id": source_object.pk,
                "source_type": get_for_model(source_object),
            })
        return super(RelatedContentQuerySet, self).filter(**kwargs)

    def by_destination(self, destination):
        """
        Retrieve any ``RelatedContent`` where the ``destination_object``
        is equal to the provided ``destination``.

        This is a shortcut for calling::

            RelatedContent.objects.filter(destination_object=destination)
        """
        return self.filter(destination_object=destination)

    def by_source(self, source):
        """
        Retrieve any ``RelatedContent`` where the ``source_object``
        is equal to the provided ``source``.

        This is a shortcut for calling::

            RelatedContent.objects.filter(source_object=source)
        """
        return self.filter(source_object=source)

    def by_type(self, type):
        """
        Shortcut for retrieving ``RelatedContent`` based on the string
        value of the ``related_type``.
        """
        return self.filter(related_type__title=type)


class RelatedContentManager(models.Manager):
    def get_query_set(self):
        return RelatedContentQuerySet(self.model, using=self._db)

    def by_destination(self, destination):
        """
        See ``RelatedContentQuerySet.by_destination``
        """
        return self.get_query_set().by_destination(destination)

    def by_source(self, source):
        """
        See ``RelatedContentQuerySet.by_source``
        """
        return self.get_query_set().by_source(source)

    def by_type(self, type):
        """
        See ``RelatedContentQuerySet.by_type``
        """
        return self.get_query_set().by_type(type)



from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey
from django.db import models
from ._utils import *

from ..models import RelatedContent
from ..models import RelatedType


def generate_model():
    one, two = generate_fake_articles(2)
    t = RelatedType.objects.create(title="Some Random Type")
    c = RelatedContent.objects.create(
        related_type=t,
        source_object=one,
        destination_object=two
    )
    return one, two, c


class RelatedTypeTestCase(TestCase):
    def test_has_title(self):
        m = RelatedType()
        self.assertModelHasField(m, "title")


class RelatedContentTestCase(TestCase):
    def generate_model(self):
        one, two, c = generate_model()
        return c

    def test_has_related_Content(self):
        m = self.generate_model()
        self.assertRelatedTo(m, "related_type", RelatedType)

    def test_has_order(self):
        m = self.generate_model()
        self.assertModelHasField(m, "order", models.IntegerField)

    def test_has_source_type(self):
        m = self.generate_model()
        self.assertRelatedTo(m, "source_type", ContentType)

    def test_has_source_id(self):
        m = self.generate_model()
        self.assertModelHasField(m, "source_id", models.PositiveIntegerField)

    def test_has_source_object(self):
        m = self.generate_model()
        self.assertTrue(hasattr(m, "source_object"))

    def test_has_destination_type(self):
        m = self.generate_model()
        self.assertRelatedTo(m, "destination_type", ContentType)

    def test_has_destination_id(self):
        m = self.generate_model()
        self.assertModelHasField(m, "destination_id",
                models.PositiveIntegerField)

    def test_has_destination_object(self):
        m = self.generate_model()
        self.assertTrue(hasattr(m, "destination_object"))


class RelatedContentFieldTestCase(TestCase):
    def test_related_contains_all_related_models(self):
        one, two, c = generate_model()
        related_content = one.related.all()
        self.assertEqual(1, related_content.count())
        self.assertEqual(related_content[0].destination_object, two)


class ReverseRelatedContentFieldTestCase(TestCase):
    def test_contains_all_related_objects_for_given_source(self):
        one, two, c = generate_model()
        related_content = two.reverse_related.all()
        self.assertEqual(1, related_content.count())
        self.assertEqual(related_content[0].source_object, one)



from django.contrib.contenttypes.generic import GenericRelation
from genericm2m.models import RelatedObjectsDescriptor

from .models import RelatedContent


class ReverseRelatedObjectsField(RelatedObjectsDescriptor):
    def __init__(self, model=None, from_field="destination_object",
            to_field="source_object"):
        if not model:
            model = RelatedContent
        super(ReverseRelatedObjectsField, self).__init__(
                model, from_field, to_field)


class RelatedContentField(GenericRelation):
    def __init__(self, **kwargs):
        defaults = {
            "object_id_field": "source_id",
            "content_type_field": "source_type",
        }
        defaults.update(kwargs)
        super(RelatedContentField, self).__init__(RelatedContent, **defaults)

from south.modelsinspector import add_ignored_fields
add_ignored_fields(["^armstrong\.apps\.related_content\.fields\.RelatedContentField"])


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'RelatedType'
        db.create_table('related_content_relatedtype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal('related_content', ['RelatedType'])

        # Adding model 'RelatedContent'
        db.create_table('related_content_relatedcontent', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('related_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['related_content.RelatedType'])),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('source_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='from', to=orm['contenttypes.ContentType'])),
            ('source_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('destination_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='to', to=orm['contenttypes.ContentType'])),
            ('destination_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('related_content', ['RelatedContent'])


    def backwards(self, orm):
        
        # Deleting model 'RelatedType'
        db.delete_table('related_content_relatedtype')

        # Deleting model 'RelatedContent'
        db.delete_table('related_content_relatedcontent')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'related_content.relatedcontent': {
            'Meta': {'ordering': "['order']", 'object_name': 'RelatedContent'},
            'destination_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'destination_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'to'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'related_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['related_content.RelatedType']"}),
            'source_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'source_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'from'", 'to': "orm['contenttypes.ContentType']"})
        },
        'related_content.relatedtype': {
            'Meta': {'object_name': 'RelatedType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['related_content']



from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from genericm2m.models import RelatedObjectsDescriptor

from . import managers


class RelatedObjectsField(RelatedObjectsDescriptor):
    def __init__(self, model=None, from_field="source_object",
            to_field="destination_object"):
        if not model:
            model = RelatedContent
        super(RelatedObjectsField, self).__init__(model, from_field, to_field)


class RelatedType(models.Model):
    title = models.CharField(max_length=100)

    def __unicode__(self):
        return self.title


class RelatedContent(models.Model):
    related_type = models.ForeignKey(RelatedType)
    order = models.IntegerField(default=0)
    source_type = models.ForeignKey(ContentType, related_name="from")
    source_id = models.PositiveIntegerField()
    source_object = generic.GenericForeignKey('source_type', 'source_id')

    destination_type = models.ForeignKey(ContentType, related_name="to")
    destination_id = models.PositiveIntegerField()
    destination_object = generic.GenericForeignKey('destination_type',
            'destination_id')

    objects = managers.RelatedContentManager()

    class Meta:
        ordering = ["order"]

    def __unicode__(self):
        return u"%s (%d): %s" % (self.related_type, self.order,
                                 self.destination_object)
